;**********************************************************************
;                                 PIC12F1501
;                              -----------U-----------  
;                        +5V --|Vdd               Vss|-- GND
;      MUTEIN (od BT modulu) --|RA5/oscin     RA0/PGD|-- LEDR out - vstup erven LED
;    MUTEOUT (do zesilovae) --|RA4/oscout    RA1/PGC|-- LEDG out - vstup zelen LED
; SWITCH - vst. tla. ON/OFF --|RA3/MCLR/Vpp  RA2/INT|-- ONSW (P-MOS gate drive)
;                              -----------------------
; fosc = 31 kHz, 1 s = 7750 tcy, 1 tcy ~ 129 s
; Spna baterie FW 1.0a - po startu ZAPNUTO
; Program vychz z programu "Spna napjen s indikac stavu baterie" PE-AR 5/2018 s.20
;**********************************************************************

	list    p=12F1501			; list directive to define processor
	#include <p12F1501.inc>		; processor specific variable definitions
  	errorlevel  -302			; suppress message 302 from list file

;	__CONFIG	_CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
	__CONFIG	_CONFIG1, _FOSC_INTOSC & _WDTE_NSLEEP & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
	__CONFIG	_CONFIG2, _WRT_OFF & _STVREN_OFF & _BORV_HI & _LPBOR_OFF & _LVP_OFF

vwarl		equ	0x40	; warning voltage (for example 3.5 V)
vwarh		equ	0x41	; warning voltage
vstpl		equ	0x42	; stop voltage (for example 3.2 V)
vstph		equ	0x43	; stop voltage
;
;       voltage: 0--------Vstp-----------Vwar----------->
;  red LED (ON): |---------|..blik..HLHL.|...svt.H...
;        system: |going OFF|...ON/OFF switch by button..
;
fadl		equ	0x70	;vyfiltrovan hodnota men 2.stupe
fadh		equ	0x71	;vyfiltrovan hodnota men 2.stupe
avel		equ	0x72	;prmr 16 poslednch men - 1.stupe filtrace
aveh		equ	0x73	;prmr 16 poslednch men - 1.stupe filtrace
wport		equ 0x75	;data na port
cycles		equ 0x76	;cykly
tmp 		equ	0x77	;cykly
func		equ	0x78	;registr mdu svcen
w_temp		equ	0x7c	; variable used for context saving 
status_temp	equ	0x7d	; variable used for context saving
;poml		equ	0x7e	;pomocn pro test
;pomh		equ	0x7f	;pomocn pro test

;numbf		equ	0x04	;poet funkc

;#define	LEDR	LATA,0		; vstup na ervenou indikan LED
;#define	LEDG	LATA,1		; vstup na zelenou indikan LED
;#define	ONSW	LATA,2		; vstup pro zen P-MOSFET (0 = ON, 1 = OFF)
;#define	MUTEOUT	LATA,4		; vstup pro zen MUTE v zesilovai (0 = MUTE)
;#define	MUTEIN	PORTA,5		; vstup MUTE od BT modulu
#define	SWITCH	PORTA,3		; ON/OFF tlatko




	ORG 0x00
	goto start

	ORG	0x004	; interrupt vector location
; Context saving for ISR
	movwf   w_temp		; save off current W register contents
	movf	STATUS,w	; move status register into W register
	movwf	status_temp	; save off contents of STATUS register
;
	btfss	INTCON,IOCIF	; interrupt from pin level change?
	goto	out			; no
	banksel	IOCAP		;(7)
	btfss	IOCAF,3		;(7) interrupt from RA3?
	goto	clearflag	; no

; odstrann zkmit tlatka
	banksel	PORTA
	call	w50f31		;50 ms pi 31 kHz
	btfsc	SWITCH
	goto	clearflag	; tlatko u nen

testsw
;	clrwdt
	incf	func,f		;
	movlw	0x02		; poet funkc - pesunout do definic -------------
	subwf	func,w
	btfsc	STATUS,Z
	clrf	func
	call	shortsel

clearflag
	banksel	IOCAP	;(7)
	clrf	IOCAF	;(7)
	banksel	PORTA

out		;nvrat s obnovou registr
	movf	status_temp,w	; retrieve copy of STATUS register
	movwf	STATUS			; restore pre-isr STATUS register contents
	swapf	w_temp,f
	swapf	w_temp,w		; restore pre-isr W register contents
	retfie					; return from interrupt

shortsel	; vbr funkce pi krtkm stisku
	movf	func,w
	andlw	0x01	; jen spodn 1 bit
	brw
	return			; func=0 vypne se v hlavn smyce
	goto	onst	; func=1 zapnuto

;-------------------------------------------------

start
; oscillator
	banksel	OSCCON	;(1)
	movlw	0x02	;LF 31 kHz
	movwf	OSCCON
; pin configuration
	banksel	LATA	;(2)
	movlw	0x04	; RA4 set to "0" (MUTE = ON), RA2 set to "1" (ONSW = 1, PWR off)
	movwf	LATA
	movlw	0x02	; CLC1OUT na RA4
	movwf	APFCON	;(2)

	banksel	TRISA	;(1)
	movlw	0x28	; RA0,RA1,RA2,RA4 output, RA3, RA5 input
	movwf	TRISA
;	bcf 	OPTION_REG,7;(1) enable WPU
	movlw	0x04	; enable WPU, TMR0 to Fosc/4, PSA on, 1:32 (asi 1 sec)
	movwf	OPTION_REG

	banksel	ANSELA	;(3)
	movlw	0x07	; digital IO
	movwf	ANSELA
	movlw	0x03	; Low-Power Sleep mode
	movwf	VREGCON

	banksel	WPUA	;(4)
	movlw	0x08	; WPU on RA3
	movwf	WPUA

; CLC configuration
	banksel	CLC1CON	;(30)
	movlw	0xc2	; CLC1 = AND, input1 = RA5
	movwf	CLC1CON
	movlw	0x01	; input1 = RA5 inverted
	movwf	CLC1GLS0
	movlw	0x00	; input2 = 0
	movwf	CLC1GLS1
	movlw	0x00	; input3 = 0
	movwf	CLC1GLS2
	movlw	0x00	; input4 = 0
	movwf	CLC1GLS3
	movlw	0x0c	; gate3 inverted, gate4 inverted, gate2 non-inverted = MUTE ON
	movwf	CLC1POL
	movlw	0x11	; CLC1IN1, TMR0IF
	movwf	CLC1SEL0
	movlw	0x50	; nem vznam, nkam mus bt pipojeno
	movwf	CLC1SEL1
	
; interrupt (switch - fall edge)
	banksel	IOCAP	;(7)
	clrf	IOCAP
	movlw	0x08	; negative edge on RA3
	movwf	IOCAN
	movlw	b'10001000'	; enable interrupts, enable IOC
	movwf	INTCON	;(core)

; initialization
	banksel	PORTA	;(0)
	clrf	TMR0
	clrf	FSR1H
	movlw	0x20	; prvn byte pole dat
	movwf	FSR1L

	movlw	0x01	; po sputn zapnuto
	movwf	func	; ON state after reset
;	call	offst	; OFF state after reset
	call	onst	; zapni

loop
;	clrwdt - je v adconvert
	movf	func,f
	btfsc	STATUS,Z	; je vypnuto (func = 0)?
	call	offst
	call	adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men
	call	volttest	; sets outputs by supply voltage
	goto	loop

;------------ podprogramy a funkce --------------

offst	; go to OFF - vypnout
	banksel	PWM2CON		;(12) zastav PWM
	clrf	PWM2CON

	banksel	CLC1CON	;(30); nastaven CLC
	movlw	0x0c	; gate3 inverted, gate4 inverted, gate non-inverted = MUTE ON
	movwf	CLC1POL

	banksel	LATA		;(2)
	bsf 	INTCON,TMR0IF

	call	w50f31		;50 ms pi 31 kHz
	call	w50f31		;50 ms pi 31 kHz
	call	w50f31		;50 ms pi 31 kHz
	call	w50f31		;50 ms pi 31 kHz
	movlw	0x04		;ONSW=1 MUTEOUT=0 LEDR=0 LEDG=0
	movwf	LATA		;(2)
;	banksel FVRCON		;(2)
	clrf	FVRCON		;(2) vypni Vref
	clrf	DACCON0		;(2) vypni DAC
	clrf	CM1CON0		;(2) vypni kompartor
	banksel	ADCON0		;(1)
	bcf 	ADCON0,ADON	;(1) vypni ADC
	banksel PORTA		;(0)
	sleep				; ...chrrrrrrrrrrrr
	return

;-----------------------------------------------------------------------------
onst	; go to ON - zapnout
	call	hefread		; ulo tabulku napt z FLASH do RAM (159 tcy i s call a return)
;ADC configuration
	banksel	ADCON0		;(1)
	movlw	b'01111101'	; ADC source = FVR, ADC is ON
	movwf	ADCON0		;(1)
	movlw	b'11110000'	; right justified, FRC oscillator, Vrpos = Vdd
	movwf	ADCON1		;(1)
	clrf	ADCON2		;(1) 0000 = No auto-conversion trigger selected
; Max. doba ustlen: kompartor 10 s, DAC 10 s, FRV 40 s, ADC ~ 5 s.
; Pi taktu 31 kHz 1 tcy ~ 129 s nen teba na ustlen ekat.
	call	adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men
	banksel	ADCON0		;(1)
; vpl pole dat (161 tcy)
	movlw	.15		; poet prvk pole
	movwf	cycles	; new
fillloop
	movf	ADRESL,w
	movwi	FSR1++
	movf	ADRESH,w
	movwi	FSR1++
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
	decfsz	cycles,f		; konec?
	goto	fillloop		; ne
;pednastaven PWM - blikn
	banksel	PWM2CON		;(12)
	clrf	PWM2DCL		;(12)
	movlw	0x80		; duty cycle 50%
	movwf	PWM2DCH		;(12)
	banksel	PORTA
	movlw	0xff		; nejdel perioda
	movwf	PR2

	clrf	TMR0	; vynulovn TMR0 - start MUTE
	bcf 	INTCON, TMR0IF
	return

;-----------------------------------------------------------------------------
w50f31	;50 ms pi 31 kHz (386 tcy, 49,8 ms)
	movlw	0x7f
	movwf	tmp
	decfsz	tmp,f		;
	goto	$-1			;
	return

;-----------------------------------------------------------------------------
adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men
	clrwdt
;zapni FVR
	banksel	LATA		;(2)
	movlw	b'10000010'	; voltage reference 2.048 V enable for ADC
	movwf	FVRCON		;(2)
;spus ADC
	banksel	ADCON0		;(1)
	bsf 	ADCON0,GO	;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO	;done/hotovo?
	goto	$-1
; zpis do pole dat (7 tcy)
	movf	ADRESL,w
	movwi	FSR1++
	movf	ADRESH,w
	movwi	FSR1++
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
; vypni DAC a FVR
	banksel	LATA		;(2)
	clrf	DACCON0		;(2)
	clrf	FVRCON		;(2)
	banksel	PORTA
; souet pole dat (163 tcy)
	clrf	avel
	clrf	aveh
	movlw	.16		; poet prvk pole
	movwf	cycles	; new
addloop
	moviw	FSR1++
	addwf	avel,f
	moviw	FSR1++
	addwfc	aveh,f
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
	decfsz	cycles,f		; konec?
	goto	addloop		; ne
; dlen 16 (8 tcy)
	swapf	avel,f
	swapf	aveh,f
	movlw	0x0f
	andwf	avel,f
	movf	aveh,w
	andlw	0xf0
	iorwf	avel,f
	xorwf	aveh,f

; 2. stupe digitln filtrace - zavd 1 bit hysterezi
;N nov hodnota - aveh:avel
;P star hodnota a vsledek - fadh:fadl
	movf	aveh,w		;Nhi
	subwf	fadh,w		;tmfh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	fi2fhi		;ne
	movf	avel,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	fadl,w		;tmfl,w		;Plo, subtract P-X
fi2fhi
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.

; Filtrace (N - nov hodnota, P - hodnota pro displej
;    N <= P -> P = N		(Z=0, C=1 - zkopruj)
;    N > P  -> P = N-1		(Z=0, C=0 - odeti 1 a zkopruj)
	btfsc	STATUS,C	;
	goto	fi2copy		;jen zkoprovn
; -1 a zkoprovn
	movlw	.1			;
	subwf	avel,w		;
	movwf	fadl		;
	movf	aveh,w		;
	btfss	STATUS,C	;
	decf	aveh,w		;
	movwf	fadh		;
	return
fi2copy	 ;zkoprovn
	movf	aveh,w		;
	movwf	fadh		;
	movf	avel,w
	movwf	fadl		;
	return

;-----------------------------------------------------------------------------
volttest	; sets outputs by supply voltage
		; vwarl		equ	0x40	; warning voltage (for example 3.75 V)
		; vwarh		equ	0x41	; warning voltage
		; vstpl		equ	0x46	; stop voltage (for example 3.5 V)
		; vstph		equ	0x47	; stop voltage

; output stop voltage (compare_unsigned_16)	; Vdd <= Vlow
	movf	fadh,w		;Nhi
	subwf	vstph,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi1	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vstpl,w		;Plo, subtract P-X
resulthi1
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.
;    N <= P -> P = N		(Z=0, C=1 - dal test)
;    N > P  -> P = N-1		(Z=0, C=0 - nastav vstupy a nvrat)
	btfsc	STATUS,C	;
	goto	tstlowvolt	; Vdd > Vstp
	clrf	func		; system go OFF - low voltage
	return

tstlowvolt	;  pokrauje test jestli je Vdd > Vlow, kdy u je Vdd > Vstp
	movf	fadh,w		;Nhi
	subwf	vwarh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi2	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vwarl,w		;Plo, subtract P-X
resulthi2
	btfsc	STATUS,C	;
	goto	powerok		; Vdd > Vwrn
	movlw	b'00010001'	; ONSW=0 (ON), MUTEOUT=1 (no), LEDG=0, LEDR=1
	movwf	wport		;(0)
;nastaven PWM - blikn erven LED
	bcf 	PIR1,TMR2IF	;clear flag (podle nvodu)
;	movlw	0x05
	movlw	0x06
	movwf	T2CON		;enable TIMER2, prescaler = 4 (5) nebo 16 (6)
	banksel	PWM2CON		;(12)
	movlw	0xd0		; PWM enable, output enable, active low
	movwf	PWM2CON		;(12)
	banksel	PORTA		;(0)
	goto	portwr

powerok	; Vdd > Vwrn
;zastav PWM
	banksel	PWM2CON		;(12) zastav PWM
	clrf	PWM2CON
	banksel	PORTA		;(0)
	clrf	T2CON		;disable TIMER2
	movlw	b'00010001'	; ONSW=0 (ON), MUTEOUT=1 (no), LEDG=0, LEDR=1
	movwf	wport		;(0)
portwr
;	bsf 	PORTA,4	;testblik
	movf	func,f		; nsl.3 instrukce zamez bliknut LED pi putn tlaka v OFF
	btfsc	STATUS,Z	; je vypnuto (func = 0)?
	return				;  ano, nech port na pokoji
	call	mutetest		;test mute tlatka
	movf	wport,w
	banksel	LATA		;(2)
	movwf	LATA		;(2)
	banksel PORTA		;(0)
	return

;-----------------------------------------------------------------------------
mutetest
	btfss	INTCON,TMR0IF	; petekl u TMR0?
	return			; ne
					; ano, nastav CLC
	banksel	CLC1CON	;(30)
	movlw	0x0e	; gate3 inverted, gate4 inverted, gate2 inverted = MUTE OFF
	movwf	CLC1POL
	return

;-----------------------------------------------------------------------------
hefread	;pete 16 byte z HEF od adresy 0x03d0 (ni byte pamovho msta)
		; a ulo je do RAM na adresy 0x40 a 0x4f
	clrf	FSR0H
	movlw	0x40
	movwf	FSR0L	; adresa 40h
	movlw   .16
	movwf	cycles
	banksel	PMADRL
	movlw	0x03
	movwf	PMADRH	; adresa H 03h
	movlw	0xd0
	movwf	PMADRL	; adresa L 80h
	bcf 	PMCON1,CFGS
hr_next
	bsf 	PMCON1,RD
	nop
	nop
	movf	PMDATL,w
	movwi	FSR0++
	incf	PMADRL,f
	decfsz  cycles,f
	goto	hr_next
	banksel PORTA
	return

;-----------------------------------------------------------------------------


	ORG 0x3d0
	DW	0x3f7b, 0x3f02	;Vwrn 027bh ~ 3.3 V
	DW	0x3fa4, 0x3f02	;Vstp 02a4h ~ 3.1 V

;	4,2 V	499	01F3
;	3,75 V	559	022F
;	3,6 V	582	0246
;	3,5 V	599	0257
;	3,4 V	616	0268
;	3,3 V	635	027B
;	3,2 V	655	028F
;	3,1 V	676	02A4
;	3,0 V	699	02BB


	ORG 0x3e0
	DE	"BOOMBOX switch MH18 www.belza.cz"

	END